@@ -164,11 +164,18 @@ class Param(ABC):
                         )
                     )
                 elif warnings:
-                    self.pcluster_config.warn(
-                        "The configuration parameter '{0}' generated the following warnings:\n{1}".format(
-                            self.key, "\n".join(warnings)
+                    if warnings[0].startswith("INFO"):
+                        print(
+                            "The configuration parameter '{0}' generated the following infos:\n{1}".format(
+                                self.key, "\n".join(warnings)
+                            )
+                        )
+                    else:
+                        self.pcluster_config.warn(
+                            "The configuration parameter '{0}' generated the following warnings:\n{1}".format(
+                                self.key, "\n".join(warnings)
+                            )
                         )
-                    )
                 else:
                     LOGGER.debug("Configuration parameter '%s' is valid", self.key)
 
@@ -215,6 +222,7 @@ class Param(ABC):
         """Reset parameter to default value."""
         self.value = self.get_default_value()
 
+    @abc.abstractmethod
     def refresh(self):
         """
         Refresh the parameter's value.
@@ -519,11 +527,11 @@ class Section(ABC):
                 errors, warnings = validation_func(self.key, self.label, self.pcluster_config)
                 if errors:
                     self.pcluster_config.error(
-                        "The section [{0}] is wrongly configured\n" "{1}".format(section_name, "\n".join(errors))
+                        "The section [{0}] is incorrectly configured\n" "{1}".format(section_name, "\n".join(errors))
                     )
                 elif warnings:
                     self.pcluster_config.warn(
-                        "The section [{0}] is wrongly configured\n{1}".format(section_name, "\n".join(warnings))
+                        "The section [{0}] is incorrectly configured\n{1}".format(section_name, "\n".join(warnings))
                     )
                 else:
                     LOGGER.debug("Section '[%s]' is valid", section_name)
