@@ -8,15 +8,8 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
-# fmt: off
-from __future__ import absolute_import, print_function  # isort:skip
 
 import functools
-
-from future import standard_library  # isort:skip
-standard_library.install_aliases()
-# fmt: on
-
 import hashlib
 import json
 import logging
@@ -141,8 +134,8 @@ def generate_random_name_with_prefix(name_prefix):
 
     Example: <name_prefix>-4htvo26lchkqeho1
     """
-    random_string = "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(16))
-    output_name = "-".join([name_prefix.lower()[: 63 - len(random_string) - 1], random_string])
+    random_string = "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(16))  # nosec
+    output_name = "-".join([name_prefix.lower()[: 63 - len(random_string) - 1], random_string])  # nosec
     return output_name
 
 
@@ -318,9 +311,11 @@ def get_supported_instance_types():
     """Return the list of instance types available in the given region."""
     ec2_client = boto3.client("ec2")
     try:
-        return [
+        supported_instance_types = set(
             offering.get("InstanceType") for offering in paginate_boto3(ec2_client.describe_instance_type_offerings)
-        ]
+        )
+        supported_instance_types.update(InstanceTypeInfo.additional_instance_types_data().keys())
+        return list(supported_instance_types)
     except ClientError as client_err:
         error(
             "Error when getting supported instance types via DescribeInstanceTypeOfferings: {0}".format(
@@ -560,27 +555,19 @@ def get_supported_os_for_scheduler(scheduler):
     :param scheduler: the scheduler for which we want to know the supported os
     :return: an array of strings of the supported OSes
     """
-    oses = ["alinux", "alinux2"]
+    oses = ["alinux2"]
     if scheduler != "awsbatch":
-        oses.extend(["centos7", "centos8", "ubuntu1604", "ubuntu1804"])
+        oses.extend(["centos7", "ubuntu1804", "ubuntu2004"])
     return list(oses)
 
 
-def get_supported_os_for_architecture(architecture):
-    """Return list of supported OSes for the specified architecture."""
-    oses = ["alinux2", "ubuntu1804", "centos8"]
-    if architecture == "x86_64":
-        oses.extend(["centos7", "alinux", "ubuntu1604"])
-    return oses
-
-
 def get_supported_schedulers():
     """
     Return a tuple of the scheduler supported by parallelcluster.
 
     :return: a tuple of strings of the supported scheduler
     """
-    return "sge", "torque", "slurm", "awsbatch"
+    return "slurm", "awsbatch"
 
 
 def get_stack_output_value(stack_outputs, output_key):
@@ -743,12 +730,11 @@ def get_installed_version():
 def check_if_latest_version():
     """Check if the current package version is the latest one."""
     try:
-        latest = json.loads(urllib.request.urlopen("https://pypi.python.org/pypi/aws-parallelcluster/json").read())[
-            "info"
-        ]["version"]
+        pypi_url = "https://pypi.python.org/pypi/aws-parallelcluster/json"
+        latest = json.loads(urllib.request.urlopen(pypi_url).read())["info"]["version"]  # nosec nosemgrep
         if packaging.version.parse(get_installed_version()) < packaging.version.parse(latest):
             print("Info: There is a newer version %s of AWS ParallelCluster available." % latest)
-    except Exception:
+    except Exception:  # nosec
         pass
 
 
@@ -1064,21 +1050,12 @@ def cluster_has_running_capacity(stack_name):
     return cluster_has_running_capacity.cached_result
 
 
-def disable_ht_via_cpu_options(instance_type, default_threads_per_core=None):
-    """Return a boolean describing whether hyperthreading should be disabled via CPU options for instance_type."""
-    if default_threads_per_core is None:
-        default_threads_per_core = InstanceTypeInfo.init_from_instance_type(instance_type).default_threads_per_core()
-    res = all(
-        [
-            # If default threads per core is 1, HT doesn't need to be disabled
-            default_threads_per_core > 1,
-            # Currently, hyperthreading must be disabled manually on *.metal instances
-            not (
-                instance_type.endswith(".metal") or instance_type.startswith("m4.") or instance_type in ["cc2.8xlarge"]
-            ),
-        ]
-    )
-    return res
+def disable_ht_via_cpu_options(instance_type):
+    """Return a boolean describing whether hyper-threading should be disabled via CPU options for instance_type."""
+    instance_info = InstanceTypeInfo.init_from_instance_type(instance_type)
+    # If default threads per core is 1, HT doesn't need to be disabled
+    # When CpuOptions is not supported, ValidThreadsPerCore can't be found in VCpuInfo info
+    return instance_info.default_threads_per_core() > 1 and 1 in instance_info.valid_threads_per_core()
 
 
 def is_hit_enabled_scheduler(scheduler):
@@ -1099,9 +1076,27 @@ def read_remote_file(url):
         if urlparse(url).scheme == "s3":
             match = re.match(r"s3://(.*?)/(.*)", url)
             bucket, key = match.group(1), match.group(2)
-            file_contents = boto3.resource("s3").Object(bucket, key).get()["Body"].read().decode("utf-8")
+            try:
+                if get_partition() == "aws":
+                    s3_resource = boto3.resource("s3", region_name="us-east-1")
+                else:
+                    s3_resource = boto3.resource("s3")
+                file_contents = s3_resource.Object(bucket, key).get()["Body"].read().decode("utf-8")
+            except ClientError as e:
+                error = e.response["Error"]
+                if error["Code"] == "IllegalLocationConstraintException":
+                    # When S3 GetObject call is made in a different Region from where the bucket exists
+                    # (e.g. bucket in an optin region while this code is in us-east-1),
+                    # make the call again by using the region from the error message, that is in the form:
+                    # "The af-south-1 location constraint is incompatible
+                    # for the Region specific endpoint this request was sent to.".
+                    match = re.match(r"The (.*) location constraint", error["Message"])
+                    s3_resource = boto3.resource("s3", region_name=match.group(1))
+                    file_contents = s3_resource.Object(bucket, key).get()["Body"].read().decode("utf-8")
+                else:
+                    raise e
         else:
-            with urllib.request.urlopen(url) as f:
+            with urllib.request.urlopen(url) as f:  # nosec nosemgrep
                 file_contents = f.read().decode("utf-8")
         return file_contents
     except Exception as e:
@@ -1117,8 +1112,12 @@ def render_template(template_str, params_dict, tags, config_version=None):
     :param params_dict: Template parameters dict
     """
     try:
-        environment = Environment(loader=BaseLoader)
-        environment.filters["sha1"] = lambda value: hashlib.sha1(value.strip().encode()).hexdigest()
+        # A nosec comment is appended to the following line in order to disable the B701 check.
+        # This is done because it's needed to enable the desired functionality. The current callers
+        # of this function pass a template_str representing either a custom template specified by
+        # the user or the default template.
+        environment = Environment(loader=BaseLoader)  # nosec nosemgrep
+        environment.filters["sha1"] = lambda value: hashlib.sha1(value.strip().encode()).hexdigest()  # nosec nosemgrep
         environment.filters["bool"] = lambda value: value.lower() == "true"
         template = environment.from_string(template_str)
         output_from_parsed_template = template.render(config=params_dict, config_version=config_version, tags=tags)
@@ -1240,11 +1239,34 @@ class Cache:
 
 
 class InstanceTypeInfo:
-    """Data object wrapping the result of a describe_instance_types call."""
+    """
+    Data object wrapping the result of a describe_instance_types call.
+
+    TODO: This class stores additional instance types data globally. In the future we should have a separate instance
+    per cluster configuration.
+    """
+
+    # Additional data used to describe instance types
+    __additional_instance_types_data = {}
 
     def __init__(self, instance_type_data):
         self.instance_type_data = instance_type_data
 
+    @staticmethod
+    def load_additional_instance_types_data(instance_types_data):
+        """Load additional data to describe instance types."""
+        InstanceTypeInfo.__additional_instance_types_data = instance_types_data if instance_types_data else {}
+
+    @staticmethod
+    def clear_additional_instance_types_data():
+        """Clear the additional data used to describe instance types."""
+        InstanceTypeInfo.__additional_instance_types_data = {}
+
+    @staticmethod
+    def additional_instance_types_data():
+        """Get the additional data used to describe instance types."""
+        return InstanceTypeInfo.__additional_instance_types_data
+
     @staticmethod
     @Cache.cached
     def init_from_instance_type(instance_type, exit_on_error=True):
@@ -1255,10 +1277,14 @@ class InstanceTypeInfo:
         The function exits with error if exit_on_error is set to True.
         """
         try:
-            ec2_client = boto3.client("ec2")
-            return InstanceTypeInfo(
-                ec2_client.describe_instance_types(InstanceTypes=[instance_type]).get("InstanceTypes")[0]
-            )
+            if instance_type in InstanceTypeInfo.additional_instance_types_data():
+                instance_type_data = InstanceTypeInfo.additional_instance_types_data()[instance_type]
+            else:
+                ec2_client = boto3.client("ec2")
+                instance_type_data = ec2_client.describe_instance_types(InstanceTypes=[instance_type]).get(
+                    "InstanceTypes"
+                )[0]
+            return InstanceTypeInfo(instance_type_data)
         except ClientError as e:
             error(
                 "Failed when retrieving instance type data for instance {0}: {1}".format(
@@ -1286,6 +1312,12 @@ class InstanceTypeInfo:
 
         return gpu_count
 
+    def gpu_type(self):
+        """Return name or type of the GPU for the instance."""
+        gpu_info = self.instance_type_data.get("GpuInfo", None)
+        # Remove space and change to all lowercase for name
+        return "no_gpu_type" if not gpu_info else gpu_info.get("Gpus")[0].get("Name").replace(" ", "").lower()
+
     def max_network_interface_count(self):
         """Max number of NICs for the instance."""
         needed_interfaces = int(self.instance_type_data.get("NetworkInfo").get("MaximumNetworkCards", 1))
@@ -1293,16 +1325,11 @@ class InstanceTypeInfo:
 
     def default_threads_per_core(self):
         """Return the default threads per core for the given instance type."""
-        # NOTE: currently, .metal instances do not contain the DefaultThreadsPerCore
-        #       attribute in their VCpuInfo section. This is a known issue with the
-        #       ec2 DescribeInstanceTypes API. For these instance types an assumption
-        #       is made that if the instance's supported architectures list includes
-        #       x86_64 then the default is 2, otherwise it's 1.
-        threads_per_core = self.instance_type_data.get("VCpuInfo", {}).get("DefaultThreadsPerCore")
-        if threads_per_core is None:
-            supported_architectures = self.instance_type_data.get("ProcessorInfo", {}).get("SupportedArchitectures", [])
-            threads_per_core = 2 if "x86_64" in supported_architectures else 1
-        return threads_per_core
+        return self.instance_type_data.get("VCpuInfo", {}).get("DefaultThreadsPerCore")
+
+    def valid_threads_per_core(self):
+        """Return the valid threads per core for the given instance type."""
+        return self.instance_type_data.get("VCpuInfo", {}).get("ValidThreadsPerCore", [])
 
     def vcpus_count(self):
         """Get number of vcpus for the given instance type."""
@@ -1321,3 +1348,24 @@ class InstanceTypeInfo:
     def is_efa_supported(self):
         """Check whether EFA is supported."""
         return self.instance_type_data.get("NetworkInfo").get("EfaSupported")
+
+    def supported_usage_classes(self):
+        """Return the list supported usage classes."""
+        supported_classes = list(self.instance_type_data.get("SupportedUsageClasses", []))
+        if "on-demand" in supported_classes:
+            # Replace official AWS with internal naming convention
+            supported_classes.remove("on-demand")
+            supported_classes.append("ondemand")
+        return supported_classes
+
+
+@Cache.cached
+def get_fsx_info(fs_id):
+    try:
+        return boto3.client("fsx").describe_file_systems(FileSystemIds=[fs_id]).get("FileSystems")[0]
+    except ClientError as e:
+        error(
+            "Failed when retrieving FSx filesystem data for filesystem {0}: {1}".format(
+                fs_id, e.response.get("Error").get("Message")
+            )
+        )
